import 'package:apex_socket/apex_socket.dart';
import 'package:flutter/material.dart';

export 'package:socket_io_client/socket_io_client.dart';

class ApexApi extends InheritedWidget {
  final ApexApiConfig config;
  static ApexSocket api;

  ApexApi(
    this.config, {
    Key key,
    @required Widget child,
  }) : super(key: key, child: child) {
    api = ApexSocket(
      config,
      androidKey: config.androidKey,
      iosKey: config.iosKey,
      webKey: config.webKey,
    );
    api.createSocket();
  }

  static ApexApi of(BuildContext context) {
    final ApexApi result = context.dependOnInheritedWidgetOfExactType<ApexApi>();
    assert(result != null, 'No ApexApi found in context');
    return result;
  }

  @override
  bool updateShouldNotify(ApexApi old) => config != old.config;

  static void privateEmit({String join, String data, bool enc}) {
    api.privateEmit(data: data, enc: enc, join: join);
  }

  static void publicEmit({String join, String data, bool enc}) {
    api.publicEmit(data: data, enc: enc, join: join);
  }

  static void privateEmitWithAck(String data, {String join, Function(String data) ack, bool enc}) {
    api.privateEmitWithAck(data, join: join, ack: ack, enc: enc);
  }

  static Future<dynamic> privateEmitWithCompleter(String data, {String join, bool enc}) {
    return api.privateEmitWithAckC(data, join: join, enc: enc);
  }

  static Future<dynamic> publicEmitWithCompleter(String data, {String join, bool enc}) {
    return api.publicEmitWithAckC(data, join: join, enc: enc);
  }
}

class KeyPair {
  final String secretKey;
  final String publicKey;

  KeyPair(this.secretKey, this.publicKey);
}

class ApexApiConfig {
  final KeyPair iosKey;
  final KeyPair webKey;
  final KeyPair androidKey;

  final String url;
  final int port;
  final OptionBuilder options;

  final String namespace;

  final String privateVersion;
  final String publicVersion;

  final String eventName;

  ApexApiConfig(
    this.url,
    this.port,
    this.namespace,
    this.eventName, {
    this.androidKey,
    this.iosKey,
    this.webKey,
    this.privateVersion = '1',
    this.publicVersion = '1',
    @required this.options,
  })  : assert(url != null && Uri.parse(url).isAbsolute, '$url must be a valid url.'),
        assert(port == null || (port >= -1 && port <= 65535),
            '$port must be a number between -1 and 65535. or null.'),
        assert(options != null, 'Options must not be null');
}
